@@ -9,7 +9,7 @@ use FFI::Platypus::Buffer;
 use FFI::Platypus::Memory;
 use Time::FFI::tm;
 
-our $VERSION = '0.002';
+our $VERSION = '1.000';
 
 our @EXPORT_OK = qw(asctime ctime gmtime localtime mktime strftime strptime);
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
@@ -116,7 +116,7 @@ $ffi->attach(strftime => ['opaque', 'size_t', 'string', 'tm'] => 'size_t' => sub
 
 $ffi->attach(strptime => ['string', 'string', 'tm'] => 'string' => sub {
   my ($xsub, $str, $format, $tm, $remaining) = @_;
-  $tm = Time::FFI::tm->new unless defined $tm;
+  $tm = Time::FFI::tm->new(tm_mday => 1, tm_isdst => -1) unless defined $tm;
   my $rc = $xsub->($str, $format, $tm);
   croak "strptime: Failed to match input to format string" unless defined $rc;
   $$remaining = $rc if defined $remaining;
@@ -205,7 +205,13 @@ L<localtime_r(3)> function is used if available.
   my $epoch = mktime $tm;
 
 Returns the epoch timestamp representing the passed L<Time::FFI::tm> record
-interpreted in the local time zone.
+interpreted in the local time zone. The time is interpreted from the L<tm_sec>,
+C<tm_min>, C<tm_hour>, C<tm_mday>, C<tm_mon>, C<tm_year>, and C<tm_isdst>
+members of the record, ignoring the rest. DST status will be automatically
+determined if C<tm_isdst> is a negative value. The C<tm_isdst>, C<tm_wday>, and
+C<tm_yday> members of the record will also be modified to be consistent with
+the interpreted time, as well as the C<tm_gmtoff> and C<tm_zone> elements if
+supported.
 
 =head2 strftime
 
@@ -224,7 +230,9 @@ for available format descriptors.
 
 Returns a L<Time::FFI::tm> record representing the passed string, parsed
 according to the passed format. Consult your system's L<strptime(3)> manual for
-available format descriptors.
+available format descriptors. The C<tm_mday> member will default to 1 if not
+specified by the string, and the C<tm_isdst> member will be set to -1; all
+other unspecified members will default to 0.
 
 A L<Time::FFI::tm> record may be passed as the third argument, in which case it
 will be modified in place to (on most systems) update only the date/time
