@@ -2,21 +2,29 @@ package Time::FFI::tm;
 
 use strict;
 use warnings;
-use Carp 'croak';
-use FFI::Platypus::Record;
-use Module::Runtime 'require_module';
-use Time::Local;
+use Carp ();
+use FFI::Platypus::Record ();
+use Module::Runtime ();
+use Time::Local ();
 
-our $VERSION = '0.002';
+our $VERSION = '1.000';
 
 my @tm_members = qw(tm_sec tm_min tm_hour tm_mday tm_mon tm_year tm_wday tm_yday tm_isdst);
 
-record_layout(
+FFI::Platypus::Record::record_layout(
   (map { (int => $_) } @tm_members),
   long   => 'tm_gmtoff',
   string => 'tm_zone',
 );
 
+sub epoch {
+  my ($self, $islocal) = @_;
+  my $year = $self->tm_year;
+  $year += 1900 if $year >= 0; # avoid timelocal/timegm year heuristic
+  my @vals = ((map { $self->$_ } qw(tm_sec tm_min tm_hour tm_mday tm_mon)), $year);
+  return $islocal ? scalar Time::Local::timelocal(@vals) : scalar Time::Local::timegm(@vals);
+}
+
 sub from_list {
   my ($class, @args) = @_;
   my %attr = map { ($tm_members[$_] => $args[$_]) } 0..$#tm_members;
@@ -30,18 +38,10 @@ sub to_list {
 
 sub to_object {
   my ($self, $class, $islocal) = @_;
-  require_module $class;
-  require Time::FFI;
+  Module::Runtime::require_module $class;
   if ($class->isa('Time::Piece')) {
-    if ($islocal) {
-      my $epoch = Time::FFI::mktime $self;
-      return $class->localtime($epoch);
-    } else {
-      my $year = $self->tm_year;
-      $year += 1900 if $year >= 0; # avoid timegm year heuristic
-      my $epoch = timegm((map { $self->$_ } qw(tm_sec tm_min tm_hour tm_mday tm_mon)), $year);
-      return $class->gmtime($epoch);
-    }
+    my $epoch = $self->epoch($islocal);
+    return $islocal ? scalar $class->localtime($epoch) : scalar $class->gmtime($epoch);
   } elsif ($class->isa('Time::Moment')) {
     my $moment = $class->new(
       year   => $self->tm_year + 1900,
@@ -52,7 +52,7 @@ sub to_object {
       second => $self->tm_sec,
     );
     return $moment unless $islocal;
-    my $epoch = Time::FFI::mktime $self;
+    my $epoch = $self->epoch(1);
     return $moment->with_offset_same_local(($moment->epoch - $epoch) / 60);
   } elsif ($class->isa('DateTime')) {
     return $class->new(
@@ -65,7 +65,21 @@ sub to_object {
       time_zone => $islocal ? 'local' : 'UTC',
     );
   } else {
-    croak "Cannot convert to unrecognized object class $class";
+    Carp::croak "Cannot convert to unrecognized object class $class";
+  }
+}
+
+sub with_extra {
+  my ($self, $islocal) = @_;
+  require Time::FFI;
+  if ($islocal) {
+    my %attr = map { ($_ => $self->$_) } qw(tm_sec tm_min tm_hour tm_mday tm_mon tm_year);
+    $attr{tm_isdst} = -1;
+    my $new = (ref $self)->new(\%attr);
+    Time::FFI::mktime($new);
+    return $new;
+  } else {
+    return Time::FFI::gmtime($self->epoch(0));
   }
 }
 
@@ -80,24 +94,33 @@ Time::FFI::tm - POSIX tm record structure
   use Time::FFI::tm;
 
   my $tm = Time::FFI::tm->new(
-    tm_year => 95, # years since 1900
-    tm_mon  => 0,  # 0 == January
-    tm_mday => 1,
-    tm_hour => 13,
-    tm_min  => 25,
-    tm_sec  => 59,
+    tm_year  => 95, # years since 1900
+    tm_mon   => 0,  # 0 == January
+    tm_mday  => 1,
+    tm_hour  => 13,
+    tm_min   => 25,
+    tm_sec   => 59,
+    tm_isdst => -1, # allow DST status to be determined by the system
   );
 
+  my $in_local = $tm->with_extra(1);
+  say $in_local->tm_isdst; # now knows if DST is active
+
   my $tm = Time::FFI::tm->from_list(CORE::localtime(time));
 
   my $epoch = POSIX::mktime($tm->to_list);
+  my $epoch = $tm->epoch(1);
 
   my $datetime = $tm->to_object('DateTime', 1);
 
 =head1 DESCRIPTION
 
 This L<FFI::Platypus::Record> class represents the C<tm> struct defined by
-F<time.h> and used by functions such as L<mktime(3)> and L<strptime(3)>.
+F<time.h> and used by functions such as L<mktime(3)> and L<strptime(3)>. This
+is used by L<Time::FFI> to provide access to such structures.
+
+The structure does not store an explicit time zone, so you must specify whether
+to interpret it as local or UTC time whenever rendering it to an actual time.
 
 =head1 ATTRIBUTES
 
@@ -124,8 +147,11 @@ F<time.h> and used by functions such as L<mktime(3)> and L<strptime(3)>.
 =head2 tm_zone
 
 The integer components of the C<tm> struct are stored as settable attributes
-that default to 0. The C<tm_gmtoff> and C<tm_zone> attributes may not be
-available on all systems. The C<tm_zone> attribute is a read-only string.
+that default to 0. Note that 0 is out of range for the C<tm_mday> value, and
+C<tm_isdst> should be set to a negative value if unknown, so these values
+should always be specified explicitly. The C<tm_gmtoff> and C<tm_zone>
+attributes may not be available on all systems. The C<tm_zone> attribute is a
+read-only string.
 
 =head1 METHODS
 
@@ -137,12 +163,22 @@ available on all systems. The C<tm_zone> attribute is a read-only string.
 
 Construct a new B<Time::FFI::tm> object representing a C<tm> struct.
 
+=head2 epoch
+
+  my $epoch = $tm->epoch($islocal);
+
+Translate the time structure into a Unix epoch timestamp (seconds since
+1970-01-01 UTC). If a true value is passed, the timestamp will represent the
+time as interpreted in the local time zone; otherwise it will be interpreted as
+UTC.
+
 =head2 from_list
 
   my $tm = Time::FFI::tm->from_list($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);
 
 Construct a new B<Time::FFI::tm> object from the passed list of values, in the
-same order returned by L<perlfunc/localtime>.
+same order returned by L<perlfunc/localtime>. Missing or undefined values will
+be interpreted as the default of 0, but see L</ATTRIBUTES>.
 
 =head2 to_list
 
@@ -158,9 +194,20 @@ L<perlfunc/localtime>.
   my $datetime = $tm->to_object('DateTime', $islocal);
 
 Return an object of the specified class. If a true value is passed as the
-second argument, the time will be interpreted in the local time zone; otherwise
-it will be interpreted as UTC. Currently L<Time::Piece>, L<Time::Moment>, and
-L<DateTime> (or subclasses) are recognized.
+second argument, the object will represent the time as interpreted in the local
+time zone; otherwise it will be interpreted as UTC. Currently L<Time::Piece>,
+L<Time::Moment>, and L<DateTime> (or subclasses) are recognized.
+
+=head2 with_extra
+
+  my $new = $tm->with_extra($islocal);
+
+Return a new B<Time::FFI::tm> object with the same time components, but with
+C<tm_wday>, C<tm_yday>, C<tm_isdst>, and (if supported) C<tm_gmtoff> and
+C<tm_zone> set appropriately. If a true value is passed, these values will be
+set according to the time as interpreted in the local time zone; otherwise they
+will be set according to the time as interpreted in UTC. Note that this does
+not replace the need to pass C<$islocal> for future conversions.
 
 =head1 BUGS
 
